
#include <stddef.h>

#include <fcntl.h>
#include <io.h>
#if !defined(SEEK_END) || \
    !defined(SEEK_SET)
# include <stdio.h>
#endif /* !SEEK_END || !SEEK_SET */
#include <share.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>

#if defined(__BORLANDC__) || \
    defined(__DMC__)
#elif defined(__INTEL_COMPILER) || \
	  defined(__MWERKS__) || \
      defined(_MSC_VER)
# define lseek	_lseek
# define open	_open
# define write	_write
#else
# error Compiler not discriminated
#endif /* compiler */


void *mmap_file(char const  *fileName   /* = NULL */
            ,   size_t      len
			,	size_t		offset
            ,   int         prot
            ,   int         flags
			,	size_t		*plen)
{
    void    *pvMap;
    int     fd;
    int     oflag;
    int     pmode;

    flags = 0; /* Ignored for the current implentation */

    if( NULL == fileName ||
        PROT_WRITE == (prot & PROT_WRITE))
    {
        prot    =   PROT_READ | PROT_WRITE;
        oflag   =   O_CREAT | O_BINARY | O_RDWR;
        pmode   =   SH_DENYRW | S_IREAD | S_IWRITE;
    }
    else
    {
        prot    =   PROT_READ;
        oflag   =   O_BINARY | O_RDONLY;
        pmode   =   SH_DENYWR | S_IREAD;
    }

    if(NULL == fileName)
    {
        flags |= MAP_ANONYMOUS;

        if(0 == len)
        {
            return MAP_FAILED;
        }
        else
        {
            size_t  pagesize    =   getpagesize();

            len = (len + pagesize - 1) & ~(pagesize - 1);
        }

        fd = -1;
    }
    else
    {
        size_t  fileLen;

#if defined(__BORLANDC__)
        fd = open(fileName, oflag);

        ((void)pmode);
#else
        fd = open(fileName, oflag, pmode);
#endif /* __SYNSOFT_DBS_DEBUG */

        if(fd < 0)
        {
            return MAP_FAILED;
        }

        fileLen = lseek(fd, 0, SEEK_END);
        lseek(fd, 0, SEEK_SET);

        if(0 == len)
        {
            len = fileLen;
        }
        else
        {
            if(prot & PROT_WRITE)
            {
                if(fileLen < len)
                {
                    lseek(fd, len - 1, SEEK_SET);
                    write(fd, "", 1);
                }
            }
        }
    }

    pvMap = mmap(NULL, len, prot, flags, fd, offset);

    if(-1 != fd)
    {
        _close(fd);
    }

	if(NULL != plen)
	{
		*plen = len;
	}

    return pvMap;
}
