/* /////////////////////////////////////////////////////////////////////////////
 * File:        dlfcn.h
 *
 * Purpose:     Declaration of the dlopen, dlclose(), dlsym() and dlerror() API
 *              functions.
 *
 * Created      1st January 2004
 * Updated:     3rd January 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/
 *                          http://www.synesis.com.au/software/
 *
 *              email:      software@synesis.com.au
 *                          software@synesis-group.com
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_DLFCN
#define _SYNSOFT_INCL_H_DLFCN

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_DLFCN_MAJOR     1
# define _SYNSOFT_VER_H_DLFCN_MINOR     0
# define _SYNSOFT_VER_H_DLFCN_REVISION	2
# define _SYNSOFT_VER_H_DLFCN_EDIT      2
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

#include <sys/types.h>

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef WIN32
# error This file is only currently defined for Win32 compilation units
#endif /* WIN32 */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants and definitions
 */

#define RTLD_LAZY	0x00001				/*!< \brief Lazy function call binding.  */
#define RTLD_NOW	0x00002				/*!< \brief Immediate function call binding.  */

/* /////////////////////////////////////////////////////////////////////////////
 * API functions
 */

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/** \brief Loads the given module, 
 *
 * \param moduleName The name of the module. The name is passed through to Win32's
 * LoadLibrary(), so the value of this parameter is interpreted according to Win32's
 * loading rules.
 * \param mode The loading mode. This is ignored on Win32 platforms, and the function
 * always behaves as if RTLD_NOW was specified
 * \note 
 */
void		*dlopen(char const *moduleName, int mode);

/** \brief Closes the given module
 *
 * \param hModule The handle of the executable module to close
 * \retval 0 success
 * \retval !0 failure. dlerror() will return an error string
 */
int			dlclose(void *hModule);

/** \brief Looks up a symbol
 *
 * \param hModule The handle of the executable module in which to search for the symbol
 * \param symbolName The name of the symbol
 * \return The pointer to the symbol, or NULL if no matching symbol was found
 * \retval NULL The symbol was not found
 */
void		*dlsym(void *hModule, char const *symbolName);

/** \brief Returns a descriptive string for the last error
 *
 * \return A pointer to a string describing the last error, or NULL.
 * \retval NULL No error occured, or dlerror() already called
 * \note Each call to this function resets it, so that a second call will always return NULL
 */
char const	*dlerror(void);

#ifdef __cplusplus
} // extern "C"
#endif /* __cplusplus */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_DLFCN */

/* ////////////////////////////////////////////////////////////////////////// */
